import alsaaudio
import logging

class AudioSource(object):
    def __init__(self,sourceId,sourceName,sourceVolume,sourceConfig,audioDev):
        self.id         = sourceId
        self.name       = sourceName
        self.volume     = sourceVolume
        self.muted      = False
        self.params     = sourceConfig
        self.audioDev   = audioDev
        self.logger     = logging.getLogger('ipspeaker')
        self.equalizer  = self.getParameter("equalizer")
        conf = self.audioDev.split(':')
        self.audioChannel = conf[1]
        if sourceVolume is not None:
            self.__setVolume()
        self.observers  = list() # hype: List[Callable(int,string,bool)]
        pass


    def playAudio(self):
        """
        Start playing audio.
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def stopAudio(self):
        """
        Mute audio
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def connect(self):
        """
        Connect audio source player
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def disconnect(self):
        """
        Disconnect audio source player
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def shutdown(self):
        """
        Shutdown audio source
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def setVolume(self, volume):
        """
        Set audio source volume
        :param volume:
        :return:
        """
        self.volume = volume
        self.__setVolume()

    def mute(self, on:bool):
        """
        Mute/unmute audio
        :param on:
        :return:
        """
        self.muted = on
        self.__setVolume()

    def __setVolume(self):
        m = alsaaudio.Mixer(self.audioChannel)
        if self.muted:
            m.setvolume(0)
        else:
            m.setvolume(int(self.volume))

    def reportStatus(self):
        """
        Report audio source status information
        :return:
        """
        raise NotImplementedError("AudioSource cannot be directly instantiated!")

    def getParameter(self,name):
        """
        Get specific parameter from audio source configuration
        :param name:
        :return:
        """
        # type: (str) -> str
        if name in self.params:
            return self.params[name]
        else:
            return ""

    def getAudioDev(self):
        return self.audioDev

    def setEqualizer(self, player):
        """
        Set initial Equalizer configuration
        :param player: player
        :return:
        """
        try:
            if "config" in self.equalizer and len(self.equalizer['config']) > 0:
                self.logger.info(f"{self.name}: Set Equalizer configuration: {self.equalizer['config']}")
                player.setEqualizer(self.equalizer['config'])
                if self.equalizer['enable'] == 'true':
                    self.logger.info(f"{self.name}: Enable Equalizer")
                    player.enableEqualizer(True)
                else:
                    self.logger.info(f"{self.name}: Equalizer is disabled (default config)")
        except Exception as e:
            self.logger.error(e)

    def setEqualizerParameter(self, player, config):
        """
        Set Equalizer Filter specific parameter (ex: central frequency, q factor or gain)
        :param player: player
        :param config: specific parameter configuration
        :return:
        """
        try:
            if "config" in self.equalizer and len(self.equalizer['config']) > 0 and any(d["name"] == config["name"] for d in self.equalizer['config']):
                if "f" in config:
                    if player is not None:
                        player.setEqualizerFrequency(config["name"], config["f"])
                    self.updateEqualizerConfig(config['name'], 'f', config['f'])
                    self.logger.info(f"{self.name}: Set Filter Central Frequency: {config}")
                elif "w" in config:
                    if player is not None:
                        player.setEqualizerQFactor(config["name"], config["w"])
                    self.updateEqualizerConfig(config['name'], 'w', config['w'])
                    self.logger.info(f"{self.name}: Set Filter Q-factor: {config}")
                elif "g" in config:
                    if player is not None:
                        player.setEqualizerGain(config["name"], config["g"])
                    self.updateEqualizerConfig(config['name'], 'g', config['g'])
                    self.logger.info(f"{self.name}: Set Filter gain: {config}")
                else:
                    self.logger.error(f"{self.name}: Set Equalizer parameter:{config}")
                    return False
                return True

            return False
        except Exception as e:
            self.logger.error(f"{self.name}: Set Equalizer parameter Exception :{e}")
            return False

    def enableEqualizer(self, player, enable):
        """
        Enable/Disable the equalizer
        :param player: player
        :param enable: enable/disable equalizer
        :return:
        """
        try:
            if player is not None:
                player.enableEqualizer(enable)
            self.equalizer['enable'] = enable
            self.logger.info(f"Equalizer enabled: {enable}")
        except Exception as e:
            self.logger.error(e)

    def updateEqualizerConfig(self, name, param, value):
        """
        Update internal equalizer config (after receiving a equalizer setting)
        :param name: band filter name
        :param param: band filter parameter
        :param value: band filter parameter value
        :return:
        """
        for filter in self.equalizer['config']:
            if filter['name'] == name:
                filter[param] = str(value)

