import socket
import time
from datetime import datetime, timedelta
import random
import logging
import threading
import shutil
import os.path
import subprocess
from urllib.error import URLError

from barix import HttpMethod


class FwUpdater:

    def __init__(self, updateUrl: str, stateFilePath: str):
        self.updateUrl = updateUrl
        self.stateFilePath = stateFilePath
        self.lock = threading.Lock()
        self.updateRunning = False
        self.timer = None
        self.startHr = None
        self.startMin = None
        self.timeWindowLen = None
        self.weekDays = None
        self.dayCheck = None
        self.triggerUpdateList = []
        self.logger = logging.getLogger("fw-updater")
        if os.path.isfile(stateFilePath):
            with open(stateFilePath, "r") as f:
                self.curFwLastModified = f.read()
        else:
            self.curFwLastModified = ""

    def setUpdateConfig(self, startTime: str, stopTime:str, weekDays: str ):
        start = datetime.strptime(startTime, "%H:%M")
        stop = datetime.strptime(stopTime, "%H:%M")
        delta = stop - start
        self.timeWindowLen = int(delta.total_seconds() / 60)
        self.weekDays = weekDays
        ( self.startHr, self.startMin ) = startTime.split(':')
        self.calculateUpdateTime()

    def calculateUpdateTime(self):
        """
        Calculate the update time for each day of the week:
         - Calculate a somewhat random hour between base and base+window_duration
         - Store the update time for each day
        """
        if self.weekDays is not None:
            randNum = random.randint(0, self.timeWindowLen)
            hrRand = randNum // 60
            mnRand = randNum % 60

            startDate = timedelta(hours=int(self.startHr), minutes=int(self.startMin), seconds=0)
            randDate = timedelta(hours=int(hrRand), minutes=int(mnRand), seconds=0)
            updateDate = str(startDate + randDate)
            (h, m, s) = updateDate.split(':')

            listDays = self.weekDays.split(',')
            for day in listDays:
                trigger = {}
                trigger['weekDay'] = time.strptime(day, "%a").tm_wday
                trigger['slot'] = self._calcWeekThresholdSlot(day, int(h), int(m))
                self.logger.info(f"Trigger: {day} at {h}:{m}")
                self.triggerUpdateList.append(trigger)

    def _calcWeekThresholdSlot(self, weekDayString, hour, min):
        """
        Calculate threshold slot to trigger firmware Update
        The week starts at Mondays ( 0 - Mon, 1 - Tue, ... 6 - Sun)
        :param weekDayString: week day string (ex: Mon, Tue, ... Sat, Sun)
        :param hour: hour
        :param min: minutes
        :return: threshold slot
        """
        weekDay = time.strptime(weekDayString, "%a").tm_wday
        return weekDay*24*3600 + hour*3600 + min*60

    def _calcCurrentWeekSlot(self):
        """
        Calculate current week Slot
        :return: current slot and weekday
        """
        now = time.localtime()
        return now.tm_wday*24*3600 + now.tm_hour*3600 + now.tm_min*60, now.tm_wday

    def checkFwUpdates(self):
        currentSlot, weekDay = self._calcCurrentWeekSlot()

        if self.dayCheck != weekDay:
            for trigger in self.triggerUpdateList:
                if trigger['weekDay'] == weekDay and currentSlot == trigger["slot"] and not self.updateRunning:
                    self.logger.info(f"Start updater: weekDay:{weekDay}, slot:{currentSlot}")
                    self.dayCheck = weekDay
                    self.spawnUpdaterThread()

    def checkAndRunUpdate(self) -> bool:
        """
        Basic check for firmware updates
         * performs HEAD to the URL and compares the Last Modified
           with the currently stored version.
         * if timestamps are found to differ, the download is
           performed and the file is installed.
        """
        try:
            self.logger.debug("checking URL: '{}'".format(self.updateUrl))
            response = HttpMethod.head(self.updateUrl, None, tlsVerify=False)
            if response.status != 200:
                self.logger.error("Cannot check firmware. HEAD returned {}".format(response.status))
                return False

            rxLastModified = response.headers.get("last-modified")
            if rxLastModified is None:
                self.logger.warning("update server doesn't provide 'last-modified'. Updating just in case...")
                return self.runUpdate()

            if rxLastModified != self.curFwLastModified:
                self.logger.info(
                    "fw 'last-modified': '{}' --> '{}'. updating".format(self.curFwLastModified, rxLastModified))
                return self.runUpdate()
            else:
                self.logger.info("no change detected: last modified is '{}'".format(self.curFwLastModified))
                return True
        except URLError as err:
            reason = err.reason
            if type(reason) is str:
                self.logger.info("URL error downloading update: {}".format(reason))
            elif type(reason) is socket.timeout:
                self.logger.info("timeout checking update")
            else:
                self.logger.info("URL error downloading update:", exc_info=err)
            return False
        except Exception as ex:
            self.logger.warning("Update finished with exception: ", exc_info=ex)
            return False

    def spawnUpdaterThread(self):
        with self.lock:
            try:
                if self.updateRunning:
                    self.logger.warning("Updater thread is already running.")
                    return
                else:
                    self.updateRunning = True
                    self.logger.info("launching firmware update task...")
                    self.timer = threading.Timer(0.5, self.updaterWorkerFunction)
                    self.timer.start()
            except Exception as ex:
                self.logger.error("Oops: some exception while spawning updater thread.", exc_info=ex)

    def updaterWorkerFunction(self):
        maxAttempts = 5
        for attempt in range(0, 5):
            self.logger.debug("running attempt {}/{}".format(attempt + 1, maxAttempts))
            if self.checkAndRunUpdate():
                self.logger.debug("sucessful check. exit.")
                with self.lock:
                    self.updateRunning = False
                    return
            else:
                self.logger.warning("update run failed. retrying in 10 seconds...")
                time.sleep(10)

        self.logger.error("gave up running update after {} failed attempts".format(maxAttempts))
        with self.lock:
            self.updateRunning = False
            return

    def runUpdate(self) -> bool:
        rxLastModified = ""
        with open("/mnt/data/fw_update_tmp.tar", "wb") as f:
            self.logger.debug("getting update from '{}'".format(self.updateUrl))
            try:
                resp = HttpMethod.get(self.updateUrl, None, tlsVerify=False)
                self.logger.info("http get completed")
            except Exception as ex:
                self.logger.error("Exception running download: ", exc_info=ex)
                return False

            try:
                if resp.status != 200:
                    self.logger.error("failed to fetch update with HTTP Status {}".format(resp.status))
                    return False

                self.logger.info("copying into destination")
                shutil.copyfileobj(resp, f)
                self.logger.info("copy finished")
                if resp.headers.get("last-modified") is not None:
                    rxLastModified = resp.headers.get("last-modified")
                else:
                    self.logger.warning("last-modified header is missing")
            except URLError as err:
                reason = err.reason
                if type(reason) is str:
                    self.logger.error("URL error downloading update: {}".format(reason))
                elif type(reason) is socket.timeout:
                    self.logger.error("timeout downloading update")
                else:
                    self.logger.error("URL error downloading update:", exc_info=reason)
                return False
            except Exception as ex:
                self.logger.error("Exception handling response: ", exc_info=ex)
                return False

        try:
            self.logger.info("running the updater...")
            ph = subprocess.Popen(["qiba-update-client", "-f", "/mnt/data/fw_update_tmp.tar"])
            ret = ph.wait()
            ret = 0
            if ret == 0:
                self.logger.info("update install successful. saving timestamp and rebooting...")
                with open(self.stateFilePath, "w") as f:
                    f.write(rxLastModified)

                subprocess.Popen("reboot")
                return True
            else:
                self.logger.info("update failed with code {}".format(ret))
                return False
        except Exception as ex:
            self.logger.error("Exception running updater: ", exc_info=ex)
            return False



