import uuid

from flask import Blueprint, Response, request
from pathlib import Path

from audio_files import *
from ipformer import getSourcesStatus

from barix.web.auth import login_required
from barix.web.exceptions import InvalidUploadedFileError, InvalidFileExtensionError, FileIsPlayingError
from barix.web.system_info import getStorageSize
from barix.web.utils import writeToJSONFile
from barix.web.audio_files import uploadAudioFile

from scheduler import updateSchedulerPlaylists

bp = Blueprint('audio_files_API', __name__)

log = logging.getLogger('flask-backend')

p = Path(AUDIO_FILES_DIR)
if not p.exists():
    try:
        p.mkdir()
    except Exception as e:
        log.error("Error creating directory {}: {}".format(AUDIO_FILES_DIR, e))

@bp.route('/api/rest/v1/ipformer/audio-files', methods=['GET'])
@login_required
def handleGetAudioFiles():
    audioFiles = {}
    try:
        if len(request.args) == 0:
            audioFileTypesRequested = ['local', 'playlists', 'usb']
        else:
            audioFileTypesRequested = request.args
        for audiofileType in audioFileTypesRequested:
            if audiofileType == "local":
                audioFiles["local"] = getLocalAudioFiles()
            elif audiofileType == "usb":
                usbAudioFiles = getUSBAudioFiles()
                if usbAudioFiles is not None: # means that there is a USB inserted
                    audioFiles["usb"] = usbAudioFiles
            elif audiofileType == "playlists":
                audioFiles["playlists"] = getPlaylists()
            else:
                return Response(status=404)
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(json.dumps(audioFiles), status=200)

@bp.route('/api/rest/v1/ipformer/audio-files', methods=['PUT'])
@login_required
def handleSaveAudioFilesDetails():
    try:
        data = json.loads(request.data)
    except Exception as e:
        log.error(e)
        return Response(status=400)
    else:
        try:
            if 'local' in data.keys():
                writeToJSONFile(AUDIO_FILES_JSON_FILE, data['local'])
            if 'playlists' in data.keys():
                playlistsDict = {}
                for elem in data['playlists']:
                    if "id" in elem.keys():
                        key = elem.pop("id")
                    else:
                        key = uuid.uuid4().hex
                    playlistsDict[key] = elem
                writeToJSONFile(PLAYLISTS_FILE_PATH, playlistsDict)
                updateSchedulerPlaylists(playlistsDict)
        except Exception as e:
            log.error(e)
            return Response(status=500)
        else:
            return Response(status=200)

@bp.route('/api/rest/v1/ipformer/audio-files', methods=['POST'])
@login_required
def handleUploadAudioFiles():
    freeSpace = getStorageSize()["available"]
    first = True
    try:
        errorFiles = []
        successCount = 0
        audioFilesList = request.files.getlist("audiofile")
        if len(audioFilesList) > 0:
            for uploadedFile in audioFilesList:
                try:
                    if not first:
                        # get free space size in storage
                        freeSpace = getStorageSize()["available"]
                    n_files_uploaded = uploadAudioFile(uploadedFile, AUDIO_FILES_DIR, AUDIO_FILES_FORMAT, freeSpace,
                                    AUDIO_FILES_JSON_FILE)
                except InvalidFileExtensionError as e:
                    log.error("Error while uploading file {}: {}".format(uploadedFile.filename, str(e)))
                    errorFiles.append("Error storing \"{}\": {}".format(uploadedFile.filename, str(e)))
                except InvalidUploadedFileError as e:
                    log.error("Error while uploading file {}: {}".format(uploadedFile.filename, str(e)))
                    errorFiles.append("Error storing \"{}\": {}".format(uploadedFile.filename, str(e)))
                except Exception as e:
                    log.error("Error while uploading file {}: {}".format(uploadedFile.filename, str(e)))
                    raise e
                else:
                    successCount += n_files_uploaded
                    first = False
        else:
            return Response("No file selected", status=400, headers={"Content-Type" : "text/plain"})
    except Exception as e:
        log.error("Error occurred while uploading file : {}".format(str(e)))
        if str(e).startswith('413 Request Entity Too Large'):
            return Response(status=413)
        elif "No space left on device" in str(e):
            return Response(status=507)
        else:
            return Response(status=500)
    else:
        return_msg = {"success" : successCount, "errors" : errorFiles}
        return Response(json.dumps(return_msg), status=200)

@bp.route('/api/rest/v1/ipformer/audio-files', methods=['DELETE'])
@login_required
def handleDeleteAudioFiles():
    try:
        data = json.loads(request.data)
    except Exception as e:
        log.error(e)
        return Response(status=400)
    else:
        try:
            nAudioFilesRemoved = deleteAudioFiles(data['payload'])
            return Response(json.dumps(nAudioFilesRemoved), status=200)
        except FileIsPlayingError:
            return Response(status=409)
        except Exception as e:
            log.error(e)
            return Response(status=500)

@bp.route('/api/rest/v1/ipformer/audio-files/play', methods=['POST'])
@login_required
def handlePlayAudioFile():
    try:
        data = json.loads(request.data)
    except Exception as e:
        log.error(e)
        return Response(status=400)
    else:
        try:
            res = playAudioFile(data['filename'])
            return Response(json.dumps(res), status=200)
        except Exception as e:
            log.error(e)
            return Response(status=500)

@bp.route('/api/rest/v1/ipformer/audio-files/stop', methods=['POST'])
@login_required
def handleStopPlayingAudioFile():
    try:
        stopPlayingAudioFile()
        return Response(status=200)
    except Exception as e:
        log.error(e)
        return Response(status=500)

@bp.route('/api/rest/v1/ipformer/audio-files/playing', methods=['GET'])
@login_required
def handleGetPlayingFile():
    filePlaying = None
    try:
        sourcesStatus = getSourcesStatus()
        if sourcesStatus is not None and "status" in sourcesStatus:
            for status in sourcesStatus["status"]:
                if status["priority"] == 0 and status["type"] == "local_file":
                    if status["state"] == "playing":
                        filePlaying = status["playing"]
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(filePlaying, status=200)