#!/usr/bin/python3
import logging

import Logger
import importlib
import json
import re
import subprocess
import alsaaudio
import signal
import sys
import time
import os
import shutil
from flask import Flask, request, Response

import baco
from audio_source_loader import *
from AudioSourceManager import AudioSourceManager
import flexa_uci
from barix.io import leds
from barix.io.Gpio import Gpio
from hwType import *
from GenericRestAPI import *
from streamer.RtpStreamer import *
from streamer.RtpOutPlayer import *
from streamer.DanteStreamer import *
from InOut import *
from FwUpdater import FwUpdater


class IPSpeaker_Mode:
    HTTP_API_IPADDRES = "127.0.0.1"
    HTTP_API_PORT = 50500

    def __init__(self, hwType):
        self.hwType = hwType
        self.logger = logging.getLogger(__name__)
        self.stopRunning    = False
        self.sourceManager  = None
        self.sourceList     = []
        self.input          = {}    # dict of digital inputs (depends on the board)
        self.relay          = {}    # dict of relays (depends on the board)
        self.nInputs        = 0
        self.nRelays        = 0
        self.enabledApiRelay = {}
        self.inputMode      = {}
        self.relaySourceList  = {}
        self.sipActivateRelay = {}
        self.dtmfStartTime  = None
        self.dtmfTimeout    = 2
        self.dtmfSequence   = ""
        self.requestToCheckDtmfTimeout = False
        self.micSelector    = None
        self.led            = None
        self.rtpStreamer    = None
        self.streamerUrl    = None
        self.rtpPlayer      = None
        self.audioInput     = None
        self.danteStreamer  = None
        self.fwUpdater      = None
        signal.signal(signal.SIGINT, self.signalHandler)
        signal.signal(signal.SIGTERM, self.signalHandler)

        self.initialize()

    def signalHandler(self, sig, frame):
        self.logger.info('SIGINT or SIGTERM received')
        self.stopRunning = True

    def runService(self, cmd,wait=False):
        p = subprocess.Popen(cmd, shell=True)
        if wait :
            p.wait()
        return p

    def startServices(self):
        #restore alsa state
        self.runService("/etc/init.d/alsa-state start", wait=True)
        pass

    def setFwUpdateConfig(self):
        fwUpdateAuto = flexa_uci.uci_get('application.fw_update.auto')
        fwUpdateUrl  = flexa_uci.uci_get('application.fw_update.url')
        startTime = flexa_uci.uci_get('application.fw_update.start_time')
        stopTime = flexa_uci.uci_get('application.fw_update.stop_time')
        weekDays = flexa_uci.uci_get('application.fw_update.week_days')
        if fwUpdateUrl is not None and fwUpdateAuto == 'true' and startTime != '' and stopTime != '' and weekDays != '':
            self.fwUpdater = FwUpdater(fwUpdateUrl,"/mnt/data/config/fw_last_modified.conf")
            self.fwUpdater.setUpdateConfig(startTime,stopTime,weekDays)

    def initialize(self):
        self.startServices()
        self.setFwUpdateConfig()
        rtpStreaming = self.checkRtpStreamer()
        danteStreaming = self.checkDanteStreamer()
        if rtpStreaming or danteStreaming:
            # update asound config and perform alsa restore
            shutil.copy('/etc/asound-streamer.conf', '/etc/asound.conf')
            self.runService("/usr/sbin/alsactl restore", wait=True)
        if rtpStreaming:
            self.startRtpStreamer()
            self.startRtpPlayer()
        if danteStreaming:
            self.startDanteStreamer()
        self.createDefaultUCISections()
        self.setFirstRunConfig()
        self.initializeMixers()
        self.initializeIOs()
        self.setAudioInput()
        # set main volume
        mainVolumeCtl = alsaaudio.Mixer("BARIX_MAIN")
        mute = flexa_uci.uci_get('application.audio.muted')
        if mute == 'true':
            mainVolumeCtl.setvolume(0)
        else:
            mainVolume = int(flexa_uci.uci_get('application.audio.volume'))
            mainVolumeCtl.setvolume(mainVolume)

        self.createPlaybackManager()

    def runMainLoop(self):
        loopCount = 0
        self.setInitEventStates()
        if self.hwType == HwType.BARIX_MPI400:
            self.sipInRelayList = {1: self.isSipInList(self.relaySrcList[1]), 2: self.isSipInList(self.relaySrcList[2])}

        if self.hwType == HwType.BARIX_MR400:
            self.sipInRelayList = {1: self.isSipInList(self.relaySrcList[1])}

        while not self.stopRunning:
            # iterate fw update checks at ~12,5 seconds
            if self.fwUpdater is not None and loopCount % 50 == 0:
                self.fwUpdater.checkFwUpdates()

            if self.hwType in [HwType.BARIX_MPI400, HwType.BARIX_MR400]:
                self.checkIOs()
            time.sleep(0.1)
        self.logger.info('Stop services')
        self.led.set("red")
        self.sourceManager.shutdown()
        self.stopRtpPlayer()
        self.stopRtpStreamer()
        self.stopDanteStreamer()

    def createDefaultUCISections(self):
        # TODO : move to IPAC run-once script
        # Create desire sections for sources. NOTE - File /barix/config/current/flexa_ip_former MUST exist!
        flexa_uci.create_uci_section('flexa_ip_former.source1')
        flexa_uci.create_uci_section('flexa_ip_former.source2')
        flexa_uci.create_uci_section('flexa_ip_former.source3')
        flexa_uci.create_uci_section('flexa_ip_former.source4')
        flexa_uci.create_uci_section('flexa_ip_former.source5')
        flexa_uci.create_uci_section('snmp.traps')

    def setFirstRunConfig(self):
        if flexa_uci.uci_get('application.main_config.run') == '':
            if self.hwType == HwType.USSI_ENC_CONTROLLER or self.hwType == HwType.HYPERSPIKE_MPA400:
                flexa_uci.uci_set('flexa_ip_former.source1.name', 'RTP Multicast')
                flexa_uci.uci_set('flexa_ip_former.source1.type', 'rtp')
                flexa_uci.uci_set('flexa_ip_former.source1.config',
                                  '{"url":"rtp://224.0.0.56:12345","playback_mode":"low_latency","equalizer": {"enable": "true", "config": [{"name": "LOW", "f": "80", "t": "q", "w": "1", "g": "0"}, {"name": "LOW-MID", "f": "500", "t": "q", "w": "1", "g": "0"}, {"name": "HI-MID", "f": "2000", "t": "q", "w": "1", "g": "0"}, {"name": "HI", "f": "4000", "t": "q", "w": "1", "g": "0"}]}}')
                flexa_uci.uci_set('flexa_ip_former.source1.volume', '90')
                flexa_uci.uci_set('flexa_ip_former.source1.prio', '1')
                flexa_uci.uci_set('flexa_ip_former.source1.enabled', 'true')
                flexa_uci.uci_commit('flexa_ip_former.source1')
            else:
                # If IPSpeaker app run for the first time it will look for Informacast and Synapps and
                # try to set them as AudioSources
                next_source_id = 1
                spec = importlib.util.find_spec("informacast")
                if spec is not None:
                    jsonFile = open('/tmp/factory_info_partition/factory_info_json', 'r')
                    config_json = json.load(jsonFile)
                    for result in config_json['SW_LICENSES']:
                        if re.search('InformaCast', result['Image_Features']):
                            logger.info("Informacast Module Found with a valid license")
                            flexa_uci.uci_set('flexa_ip_former.source1.name', 'Informacast')
                            flexa_uci.uci_set('flexa_ip_former.source1.type', 'informacast')
                            flexa_uci.uci_set('flexa_ip_former.source1.config',
                                              '{"url":"","idleaudio":"","idleaudio_format":"ULAW_8KHZ"}')
                            flexa_uci.uci_set('flexa_ip_former.source1.volume', '90')
                            flexa_uci.uci_set('flexa_ip_former.source1.prio', '1')
                            flexa_uci.uci_set('flexa_ip_former.source1.enabled', 'true')
                            flexa_uci.uci_commit('flexa_ip_former.source1')
                            next_source_id = 2
                        else:
                            self.logger.info("Informacast Module Found but no valid license")

                spec = importlib.util.find_spec("synapps")
                if spec is not None:
                    self.logger.info("Synapps Module Found")
                    flexa_uci.uci_set('flexa_ip_former.source{}.type'.format(next_source_id), 'syn-apps')
                    flexa_uci.uci_set('flexa_ip_former.source{}.enabled'.format(next_source_id), 'true')
                    flexa_uci.uci_set('flexa_ip_former.source{}.volume'.format(next_source_id), '90')
                    flexa_uci.uci_set('flexa_ip_former.source{}.prio'.format(next_source_id),
                                      '{}'.format(next_source_id))
                    flexa_uci.uci_set('flexa_ip_former.source{}.name'.format(next_source_id), 'Synapps')
                    flexa_uci.uci_set('flexa_ip_former.source{}.config'.format(next_source_id), '{"server":""}')
                    flexa_uci.uci_commit('flexa_ip_former.source{}'.format(next_source_id))
            # create UCI
            flexa_uci.uci_set('application.main_config.run', 'true')
            flexa_uci.uci_commit('application.main_config.run')

    def initializeMixers(self):
        logger.info("Initialize Mixers")
        alsaaudio.PCM(device='channel1')
        alsaaudio.PCM(device='channel2')
        alsaaudio.PCM(device='channel3')
        alsaaudio.PCM(device='channel4')
        alsaaudio.PCM(device='channel5')
        alsaaudio.PCM(device='channel11')
        alsaaudio.PCM(device='channel12')

    def initializeIOs(self):
        if self.hwType == HwType.BARIX_MPI400:
            self.initializeMPI400_IOs()
        if self.hwType == HwType.BARIX_MR400:
            self.initializeMR400_IOs()

    def initializeMPI400_IOs(self):
        self.logger.info("Initializing IOs for MPI400")
        self.micSelector = Gpio("410", Gpio.OUT)
        # INPUT 1 and 2
        self.input.update({1: Input(413)})
        self.input.update({2: Input(412)})
        self.nInputs = 2
        self.inputMode.update({1: f"{flexa_uci.uci_get('application.io.input1_op_mode')}"})
        self.inputMode.update({2: f"{flexa_uci.uci_get('application.io.input2_op_mode')}"})
        self.logger.info(f"INPUTs Mode - {self.inputMode}")
        # RELAY 1 and 2
        self.relay.update({1: Relay(414)})
        self.relay.update({2: Relay(415)})
        self.nRelays = 2

        if flexa_uci.uci_get("application.io.api_relay1") == "true":
            self.enabledApiRelay.update({1: True})
        else:
            self.enabledApiRelay.update({1: False})
        if flexa_uci.uci_get("application.io.api_relay2") == "true":
            self.enabledApiRelay.update({2: True})
        else:
            self.enabledApiRelay.update({2: False})

        self.relaySourceList.update({1: f"{flexa_uci.uci_get('application.io.relay1_source_list')}"})
        self.relaySourceList.update({2: f"{flexa_uci.uci_get('application.io.relay2_source_list')}"})
        self.logger.info(f"RELAYs Source List - {self.relaySourceList}")
        self.sipActivateRelay.update({1: f"{flexa_uci.uci_get('application.io.sip_activate_relay1')}"})
        self.sipActivateRelay.update({2: f"{flexa_uci.uci_get('application.io.sip_activate_relay2')}"})
        self.logger.info(f"RELAYs Sip Activation - {self.sipActivateRelay}")
        # optional IO sip related settings
        self.autodial = {1: flexa_uci.uci_get('application.io.sip_autodial1'),
                         2: flexa_uci.uci_get('application.io.sip_autodial2')}
        self.logger.info(f"SIP input autodial - {self.autodial}")
        self.blindTransferExt = {1: flexa_uci.uci_get('application.io.sip_blind_transfer_ext1'),
                                 2: flexa_uci.uci_get('application.io.sip_blind_transfer_ext2')}
        self.logger.info(f"SIP blind transfer - {self.blindTransferExt}")
        self.relayDtmfActivate = {1: flexa_uci.uci_get('application.io.sip_relay1_dtmf_activate'),
                                  2: flexa_uci.uci_get('application.io.sip_relay2_dtmf_activate')}
        self.logger.info(f"SIP DTMF activate relay - {self.relayDtmfActivate}")
        self.relayDtmfDeactivate = {1: flexa_uci.uci_get('application.io.sip_relay1_dtmf_deactivate'),
                                    2: flexa_uci.uci_get('application.io.sip_relay2_dtmf_deactivate')}
        self.logger.info(f"SIP DTMF deactivate relay - {self.relayDtmfDeactivate}")
        self.relaySrcList = {1: flexa_uci.uci_get('application.io.relay1_source_list'),
                             2: flexa_uci.uci_get('application.io.relay2_source_list')}
        self.logger.info(f"Relay Source List - {self.relaySrcList}")

    def initializeMR400_IOs(self):
        self.logger.info("Initializing IOs for MR400")
        self.nInputs = 0
        # RELAY 1 and 2
        self.relay.update({1: Relay(414)})
        self.nRelays = 1

        if flexa_uci.uci_get("application.io.api_relay1") == "true":
            self.enabledApiRelay.update({1: True})
        else:
            self.enabledApiRelay.update({1: False})

        self.relaySourceList.update({1: f"{flexa_uci.uci_get('application.io.relay1_source_list')}"})
        self.logger.info(f"RELAYs Source List - {self.relaySourceList}")
        self.sipActivateRelay.update({1: f"{flexa_uci.uci_get('application.io.sip_activate_relay1')}"})
        self.logger.info(f"RELAYs Sip Activation - {self.sipActivateRelay}")
        # optional IO sip related settings
        self.blindTransferExt = {1: flexa_uci.uci_get('application.io.sip_blind_transfer_ext1')}
        self.logger.info(f"SIP blind transfer - {self.blindTransferExt}")
        self.relayDtmfActivate = {1: flexa_uci.uci_get('application.io.sip_relay1_dtmf_activate')}
        self.logger.info(f"SIP DTMF activate relay - {self.relayDtmfActivate}")
        self.relayDtmfDeactivate = {1: flexa_uci.uci_get('application.io.sip_relay1_dtmf_deactivate')}
        self.logger.info(f"SIP DTMF deactivate relay - {self.relayDtmfDeactivate}")
        self.relaySrcList = {1: flexa_uci.uci_get('application.io.relay1_source_list')}
        self.logger.info(f"Relay Source List - {self.relaySrcList}")

    def setInitEventStates(self):
        if self.hwType == HwType.BARIX_MPI400:
            # SIP EVENTS
            self.relaySipPreviousState = [SipCallState.IDLE,SipCallState.IDLE]

    def setAudioInput(self):
        self.audioInput = flexa_uci.uci_get("application.audio.input")
        micInGain = flexa_uci.uci_get("application.audio.micin_gain")
        lineInGain = flexa_uci.uci_get("application.audio.linein_gain")
        self.logger.info(f"Audio Input: {self.audioInput}, MIC Gain:{micInGain}, Line-In Gain:{lineInGain}")
        if self.audioInput == "LINE_IN":
            self.setMixerParam("Line In", "cap")
            self.setMixerParam("Line In", f"{lineInGain}%")
            self.setMixerParam("ADC Gain", "0dB")
            self.setMixerParam("Mic1", "nocap")
            #self.micSelector.write(1)
        elif self.audioInput == "MIC_IN" or self.audioInput == "PHANTOM_MIC_IN":
            if self.audioInput == "MIC_IN" and self.micSelector is not None:
                self.micSelector.write(1)
            elif self.micSelector is not None:
                self.micSelector.write(0)
            #common settings for both MICs
            self.setMixerParam("Mic1", "cap")
            self.setMixerParam("Mic1", "4.5dB")
            self.setMixerParam("Mic1 Boost", str(str(micInGain) + "dB"))
            self.setMixerParam("Line In", "nocap")
        elif self.audioInput == 'DANTE':
            pass
        elif self.audioInput == 'NONE':
            self.setMixerParam("ADC Gain", "0dB")
            self.setMixerParam("Mic1", "nocap")
            self.setMixerParam("Line In", "nocap")
        else:
            self.logger.error("Invalid audio input")

    def setMixerParam(self, param, value):
        p = subprocess.Popen("amixer -q -c 2 -- sset " + "\'" + param + "\'" + " \'" + value + "\'", shell=True)
        p.wait()

    def getSipSource(self):
        for src in self.sourceList:
            if src['type'] == 'sip':
                return src
        return None

    def isSipSource(self, index):
        for src in self.sourceList:
            if src['sourceId'] == index and src['type'] == 'sip':
                return True
        return False

    def isSipInList(self, list):
        result = json.loads(list)
        for index in result:
            for src in self.sourceList:
                if src['sourceId'] == index and src['type'] == 'sip':
                    return True
        return False

    def inputSipOpMode(self, inputId):
        sipSource = self.getSipSource()
        if sipSource is not None:
            currentSipState = sipSource['source'].reportSipCallState()
            autodial = None
            if self.autodial[inputId] != '':
                autodial = self.autodial[inputId]

            autoAnswer = False
            if "auto_answer" in sipSource['sourceCfg']:
                autoAnswer = sipSource['sourceCfg']['auto_answer']

            if currentSipState == SipCallState.IDLE and autodial is not None:
                self.logger.info(f"SIP AUTODIAL {inputId}: calling {autodial}")
                sipSource['source'].makeCall(autodial)

            if currentSipState == SipCallState.RINGING and not autoAnswer:
                self.logger.info(f"SIP RINGING + INPUT{inputId} --> ANSWER")
                sipSource['source'].answerCall()

            if currentSipState != SipCallState.IDLE and currentSipState != SipCallState.RINGING:
                self.logger.info(f"SIP is not IDLE and not receiving a call + INPUT{inputId} --> HANGUP")
                sipSource['source'].hangupCall()

    def inputSipPhrOpMode(self, inputId, inputState):
        sipSource = self.getSipSource()
        if sipSource is not None:
            currentSipState = sipSource['source'].reportSipCallState()
            autodial = None
            if self.autodial[inputId] != '':
                autodial = self.autodial[inputId]

            autoAnswer = False
            if "auto_answer" in sipSource['sourceCfg']:
                autoAnswer = sipSource['sourceCfg']['auto_answer']

            if inputState == Input.PRESSED:
                if currentSipState == SipCallState.IDLE and autodial is not None:
                    self.logger.info(f"SIP AUTODIAL {inputId}: calling {autodial}")
                    sipSource['source'].makeCall(autodial)

                if currentSipState == SipCallState.RINGING and not autoAnswer:
                    self.logger.info(f"SIP RINGING + INPUT{inputId} --> ANSWER")
                    sipSource['source'].answerCall()
            else: # INPUT Released
                if currentSipState != SipCallState.IDLE and currentSipState != SipCallState.RINGING:
                    self.logger.info(f"SIP is not IDLE and not receiving a call + INPUT{inputId} --> HANGUP")
                    sipSource['source'].hangupCall()

    def inputSipTransferOpMode(self, inputId, inputState):
        sipSource = self.getSipSource()
        if sipSource is not None:
            currentSipState = sipSource['source'].reportSipCallState()
            transferExtension = None
            if self.blindTransferExt[inputId] != '':
                transferExtension = self.blindTransferExt[inputId]
            
            if inputState == Input.PRESSED:
                if ( currentSipState == SipCallState.CONNECTED or currentSipState == SipCallState.DTMF_DIGITS ) and transferExtension is not None:
                    self.logger.info(f"SIP Blind Transfer on input {inputId}: transfer extension {transferExtension}")
                    sipSource['source'].transferCall(transferExtension)

    def relayOnSip(self, relayId, activate):
        sipSource = self.getSipSource()
        if sipSource is not None:
            currentSipState = sipSource['source'].reportSipCallState()
            relayState = self.relay[relayId].getState()

            #if self.relaySipPreviousState[relayId-1] != currentSipState:

            if activate == SIP_ACTIVATE_RELAY.ON_SIP_RINGING and currentSipState == SipCallState.RINGING and relayState == Relay.OFF:
                self.relay[relayId].write(1)
                self.logger.info(f"Relay {relayId} ON - on sip ringing")

            if activate == SIP_ACTIVATE_RELAY.ON_SIP_DTMF and currentSipState == SipCallState.DTMF_DIGITS and self.relayDtmfActivate[relayId] != '' and self.relayDtmfDeactivate[relayId] != '' :
                digit = sipSource['source'].reportDtmfDigit()
                if digit != '':
                    self.dtmfSequence += digit
                    self.logger.info(f"New DTMF sequence:{self.dtmfSequence}")
                    self.dtmfStartTime = time.monotonic()
                    self.requestToCheckDtmfTimeout = True

                if relayState == Relay.OFF and self.relayDtmfActivate[relayId] != '':
                    relayDtmfActivate = self.relayDtmfActivate[relayId]
                    if relayDtmfActivate == self.dtmfSequence:
                        self.relay[relayId].write(1)
                        self.logger.info(f"Relay {relayId} ON - on sip DTMF")

                if relayState == Relay.ON and self.relayDtmfDeactivate[relayId] != '':
                    relayDtmfDeactivate = self.relayDtmfDeactivate[relayId]
                    if relayDtmfDeactivate == self.dtmfSequence:
                        self.relay[relayId].write(0)
                        self.logger.info(f"Relay {relayId} OFF - on sip DTMF")

        #        currentSipState = 99 # SIP DTMF_DIGITS state will stay until the call is disconnected - so we must force here to a different state to retrieve DTMF digits

            if activate == SIP_ACTIVATE_RELAY.ON_SIP_ACTIVE_CALL and currentSipState == SipCallState.CONNECTED and relayState == Relay.OFF:
                self.relay[relayId].write(1)
                self.logger.info(f"Relay {relayId} ON - on sip active call")

            if currentSipState == SipCallState.IDLE and relayState == Relay.ON:
                self.relay[relayId].write(0)
                self.logger.info(f"Relay {relayId} OFF - on SIP IDLE")

            if self.requestToCheckDtmfTimeout:
                self.checkDtmfSeqTimeout()
            #self.relaySipPreviousState[relayId-1] = currentSipState

    def checkDtmfSeqTimeout(self):
        if ( time.monotonic() - self.dtmfStartTime ) >= self.dtmfTimeout:
            self.logger.info(f"dtmf timeout of {self.dtmfTimeout} seconds reached, will clean dtmf seq list")
            self.dtmfSequence = ""
            self.dtmfStartTime = None
            self.requestToCheckDtmfTimeout = False

    def relayOnPlaying(self, relayId, playingSrc):
        try:
            relayState = self.relay[relayId].getState()
            if playingSrc is not None:
                if playingSrc in self.relaySrcList[relayId]:
                    if relayState == Relay.OFF:
                        self.relay[relayId].write(1)
                        self.logger.info(f"ON Playing -> Relay {relayId} ON - source {playingSrc}")
                else:
                    if relayState == Relay.ON:
                        self.relay[relayId].write(0)
                        self.logger.info(f"ON playing -> Relay {relayId} OFF")
            else:
                if relayState == Relay.ON:
                    self.relay[relayId].write(0)
                    self.logger.info(f"ON playing -> Relay {relayId} OFF")

        except Exception as e:
            self.logger.error(f"Exception relayOnPlaying: {e}")

    def checkIOs(self):
        try:
            for id in range(1,self.nInputs+1):
                evt = self.input[id].checkEvents()
                mode = self.inputMode[id]
                if evt == Input.PRESSED:
                    if mode == INPUT_MODE.SIP:
                        self.inputSipOpMode(id)
                    if mode == INPUT_MODE.SIP_PHR:
                        self.inputSipPhrOpMode(id, evt)
                    if mode == INPUT_MODE.SIP_REFER_BLIND_TRANSFER:
                        self.inputSipTransferOpMode(id, evt)

                if evt == Input.RELEASED:
                    if mode == INPUT_MODE.SIP_PHR:
                        self.inputSipPhrOpMode(id, evt)

            for id in range(1,self.nRelays+1):
                if len(self.relaySourceList[id]):             
                    if not self.enabledApiRelay[id]: #only sets specific relay action if API Control is not enabled
                        if self.sourceManager is not None:
                            playingSrc = self.sourceManager.getPlayingSrc()
                            if playingSrc is not None:
                                if not self.isSipSource(int(playingSrc)):
                                    self.relayOnPlaying(id, str(playingSrc))
                                elif self.sipInRelayList[id]:
                                        self.relayOnSip(id, self.sipActivateRelay[id])
                            else: # no source playing => use relayOnPlaying to turn off the relay (only if is ON)
                                self.relayOnPlaying(id, playingSrc)

        except Exception as e:
            self.logger.error(f"Check IOs: {e}")

    def getInputsState(self):
        resp = {}
        for id in range(1, self.nInputs+1):
            if self.input[id].getState() == Input.PRESSED:
                resp[f"input{id}"] = '1'
            else:
                resp[f"input{id}"] = '0'
        return resp

    def getRelaysState(self):
        resp = {}
        for id in range(1, self.nRelays+1):
            if self.relay[id].getState() == Relay.ON:
                resp[f"relay{id}"] = '1'
            else:
                resp[f"relay{id}"] = '0'
        return resp

    def setRelayByAPI(self, relayId, enable):
        if flexa_uci.uci_get(f"application.io.api_relay{relayId}") == "true":
            self.relay[relayId].write(enable)
            return True
        else:
            return False

    def createPlaybackManager(self):
        self.sourceManager = AudioSourceManager()
        self.loadAudioSources()

    def start(self, flaskApp):
        self.sourceManager.start()
        self.led = leds.IpamLed("led2")
        self.led.set("green")
        self.flaskApp = flaskApp
        self.runFlaskThread()
        self.runMainLoop()

    def loadAudioSources(self):
        self.logger.info("Loading audio source manager...")
        # Local Audio Source configuration - used to play local audio files through webUI play button
        source = load_audio_source('local_file', 0, "Play Local Audio File", None, {"port":60000,"loop":"false"}, "plug:master1")
        self.sourceManager.addPlayer(0,'local_file',source)
        self.logger.info("Loaded source: id=0, name=Play Local Audio File, type=local_file")

        for sourceId in range (1, 6):
            baseVar = f"flexa_ip_former.source{sourceId}"
            sourceEnabled = flexa_uci.uci_get(baseVar + ".enabled")
            if sourceEnabled != "true":
                self.logger.info(f"Source {sourceId} is not enabled ({sourceEnabled}). Skipping...")
                print(f"Source {sourceId} is not enabled ({sourceEnabled}). Skipping...")
                continue

            sourceName = flexa_uci.uci_get(baseVar + ".name")
            sourcePrio = int(flexa_uci.uci_get(baseVar + ".prio"))
            sourceType = flexa_uci.uci_get(baseVar + ".type")
            if sourceType == "":
                self.logger.error(f"Source {sourceId} has empty type!")
                continue

            sourceVolume = flexa_uci.uci_get(baseVar + ".volume")
            if sourceVolume == "":
                self.logger.error(f"Source {sourceId} has empty volume!")
                continue

            configStr   = flexa_uci.uci_get(baseVar + ".config")
            self.logger.debug(f"IPF: loading config from string:'{configStr}'")
            jsonConfig  = json.loads(configStr)
            audioDev = "plug:chan_" + str(sourceId)
            if sourceType == "scheduler":
                audioDev = ["plug:chan_11", "plug:chan_12"]
            self.logger.info(f"Audio Source will use audio channel:'{audioDev}'")

            if sourceType == 'line-in': #only for MPI400
                source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev,"plug:dsnoop_analog")
            elif sourceType == 'sip':
                if self.audioInput == 'DANTE':
                    source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev,"plug:loopback2_in")
                elif self.audioInput != 'NONE' and self.hwType == HwType.BARIX_MPI400:
                    source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev,"plug:dsnoop_analog")
                else:
                    source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev)
            elif sourceType == 'dante':
                source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev,"plug:loopback2_in")
            else:
                source = load_audio_source(sourceType, sourceId, sourceName, sourceVolume, jsonConfig, audioDev)

            self.logger.info("Loaded source: id={}, name=\"{}\", type={}".format(sourceId, sourceName, sourceType))
            self.sourceManager.addPlayer(sourcePrio,sourceType,source)
            self.sourceList.append({"sourceId": sourceId, "source": source, "type":sourceType, "sourceCfg": jsonConfig})

    def checkRtpStreamer(self):
        """
        Check if Rtp Streamer is enabled/disabled
        :return: bool
        """
        if flexa_uci.uci_get("application.rtp_out.enabled") != "true":
            self.logger.info("RTP streamer feature disabled")
            return False
        self.logger.info("RTP streamer feature enabled")
        return True

    def startRtpStreamer(self):
        self.streamerUrl = flexa_uci.uci_get("application.rtp_out.url")
        audioFormat = flexa_uci.uci_get("application.rtp_out.audio_format")
        if self.streamerUrl == '' or audioFormat == '':
            self.logger.error(f"RTP streamer: missing parameters-> url:{self.streamerUrl}, audio format:{audioFormat}")
        else:
            self.rtpStreamer = RtpStreamer("plug:loopback_in",self.streamerUrl,audioFormat)
            if self.rtpStreamer.loadProfileParams():
                self.rtpStreamer.startStreamer()

    def stopRtpStreamer(self):
        if self.rtpStreamer is not None:
            self.logger.info("RTP Streamer stop")
            self.rtpStreamer.stopStreamer()

    def startRtpPlayer(self):
        if self.streamerUrl is not None:
            playbackDev = "plug:analog_out"
            if self.hwType in [HwType.BARIX_TPA400, HwType.BARIX_MPA400, HwType.HYPERSPIKE_MPA400, HwType.USSI_ENC_CONTROLLER] :
                playbackDev = "plug:digital_out"
            if flexa_uci.uci_get("application.rtp_out.playback") != "regular":
                self.logger.info("Using RTP Low Latency playback to play from the streamed RTP")
                self.rtpPlayer = RtpOutPlayer(self.streamerUrl,playbackDev,"low_latency")
            else:
                self.rtpPlayer = RtpOutPlayer(self.streamerUrl,playbackDev,"regular")
            self.rtpPlayer.startPlayer()
        else:
            logger.error("Cannot start player -> Invalid streamer URL")

    def stopRtpPlayer(self):
        if self.rtpPlayer is not None:
            self.logger.info("Rtp Player stop")
            self.rtpPlayer.stopPlayer()

    def checkDanteStreamer(self):
        """
        Check if Dante Streamer is enabled/disabled
        :return: bool
        """
        if flexa_uci.uci_get("application.dante.streaming") != "true":
            self.logger.info("Dante streamer feature disabled")
            return False
        self.logger.info("Dante streamer feature enabled")
        return True

    def startDanteStreamer(self):
        self.danteStreamer = DanteStreamer()
        self.danteStreamer.start()

    def stopDanteStreamer(self):
        if self.danteStreamer is not None:
            self.danteStreamer.stop()

    def runFlaskThread(self):
        flaskthread = Thread(target=self.runFlask)
        flaskthread.daemon = True
        flaskthread.start()

    def runFlask(self):
        self.logger.info("Running Internal API")
        self.flaskApp.run(host=IPSpeaker_Mode.HTTP_API_IPADDRES, port=IPSpeaker_Mode.HTTP_API_PORT)

    def isSourceEnabled(self, id):
        baseVar = f"flexa_ip_former.source{id}"
        sourceEnabled = flexa_uci.uci_get(baseVar + ".enabled")
        if sourceEnabled != "true":
            return False
        return True

    def setSourceEqualizer(self, config):
        if self.isSourceEnabled(config['sourceId']):
            for src in self.sourceList:
                if src['sourceId'] == config['sourceId']:
                    return src['source'].setEqualizerParameter(config["config"])
        else:
            self.logger.error(f"Cannot set equalizer config because audio source{config['sourceId']} is disabled")
        return False

    def enableEqualizer(self, config):
        if self.isSourceEnabled(config['sourceId']):
            for src in self.sourceList:
                if src['sourceId'] == config['sourceId']:
                    src['source'].enableEqualizer(config["enable"])
                    return True
        else:
            self.logger.error(f"Cannot enable/disable equalizer because audio source{config['sourceId']} is disabled")
        return False

    def setScheduler(self, config):
        if self.isSourceEnabled(config['sourceId']):
            for src in self.sourceList:
                if src['sourceId'] == config['sourceId']:
                    src['source'].setSchedulerConfig()
                    return True
        else:
            self.logger.error(f"Cannot restart scheduler because audio source{config['sourceId']} is disabled")
        return False

# ------------------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------------------

if __name__ == "__main__":
    Logger.initLogger('/var/log/ipac.log', 2 * 1024 * 1024, 3, False)
    logger = logging.getLogger(__name__)
    logger.info("IP Audio Client Starting -- ipspeaker mode")

    if len(sys.argv) != 2:
        logger.error("need to receive hwType argument")
        sys.exit(1)

    hwType = int(sys.argv[1])
    logger.info("Running on device type:{}".format(hwType))

    baco.initializeLogger("/var/log/baco.log", baco.LogPriority.INFO, 5000000, 2, False)
    baco.initializeAudioLib()

    stdAPP = IPSpeaker_Mode(hwType)

    app = Flask(__name__)

    @app.route('/api/equalizer/config', methods=['PUT'])
    def setEqualizer():
        try:
            cmd = json.loads(request.data)
            if "sourceId" in cmd and "config" in cmd and type(cmd["sourceId"]) == int:
                if (cmd["sourceId"] >= 1) and (cmd["sourceId"] <= 5):
                    if stdAPP.setSourceEqualizer(cmd):
                        return Response("Ok", status=200)
                    else:
                        return Response("Parameter Error/Missing", status=400)
                else:
                    return Response("Parameter Error", status=400)
            else:
                return Response("Parameter Error", status=400)
        except Exception as e:
            logger.error(e)
            return Response(status=500)

    @app.route('/api/equalizer/enable', methods=['PUT'])
    def enableEqualizer():
        try:
            cmd = json.loads(request.data)
            if "sourceId" in cmd and "enable" in cmd and type(cmd["sourceId"]) == int:
                if (cmd["sourceId"] >= 1) and (cmd["sourceId"] <= 5):
                    if stdAPP.enableEqualizer(cmd):
                        return Response("Ok", status=200)
                    else:
                        return Response("Parameter Error/Missing", status=400)
                else:
                    return Response("Parameter Error", status=400)
            else:
                return Response("Parameter Error", status=400)
        except Exception as e:
            logger.error(e)
            return Response(status=500)

    @app.route('/api/scheduler/set', methods=['PUT'])
    def setScheduler():
        try:
            cmd = json.loads(request.data)
            if "sourceId" in cmd and type(cmd["sourceId"]) == int:
                if (cmd["sourceId"] >= 1) and (cmd["sourceId"] <= 5):
                    if stdAPP.setScheduler(cmd):
                        return Response("Ok", status=200)
                    else:
                        return Response("Parameter Error/Missing", status=400)
                else:
                    return Response("Parameter Error", status=400)
            else:
                return Response("Parameter Error", status=400)
        except Exception as e:
            logger.error(e)
            return Response(status=500)

    @app.route('/api/inputs/get', methods=['GET'])
    def getInputsState():
        try:
            return Response(json.dumps(stdAPP.getInputsState()), status=200, headers={"content-type": "application/json"})
        except Exception as e:
            logger.error(e)
            return Response(status=500)


    @app.route('/api/relays/get', methods=['GET'])
    def getRelaysState():
        try:
            return Response(json.dumps(stdAPP.getRelaysState()), status=200, headers={"content-type": "application/json"})
        except Exception as e:
            logger.error(e)
            return Response(status=500)


    @app.route('/api/relay/set', methods=['PUT'])
    def setRelay():
        try:
            cmd = json.loads(request.data)
            keys = list(cmd.keys())
            if len(keys) != 1:
                return Response(json.dumps({"result": "INVALID_PARAMETERS"}), status=400, headers={"Content-Type": "application/json"})

            if "relay" not in keys[0]:
                return Response(json.dumps({"result": "INVALID_PARAMETERS"}), status=400, headers={"Content-Type": "application/json"})

            name, relayId = keys[0].split("relay")
            if name != '' or not relayId.isnumeric():
                return Response(json.dumps({"result": "INVALID_PARAMETERS"}), status=400, headers={"Content-Type": "application/json"})

            relayId = int(relayId)
            if relayId < 1 or relayId > stdAPP.nRelays:
                return Response(json.dumps({"result": "INVALID_PARAMETERS"}), status=400, headers={"Content-Type": "application/json"})

            if cmd[keys[0]] not in ["0", "1"]:
                return Response(json.dumps({"result": "INVALID_PARAMETERS"}), status=400, headers={"Content-Type": "application/json"})

            enable = int(cmd[keys[0]])
            if stdAPP.setRelayByAPI(relayId, enable):
                return Response(json.dumps({"result": "OK"}), status=200, headers={"Content-Type": "application/json"})
            else:
                return Response(json.dumps({"result": "NOT_ALLOWED"}), status=403, headers={"Content-Type": "application/json"})

        except Exception as e:
            logger.error(e)
            return Response(status=500)

    stdAPP.start(app)
