#!/usr/bin/python3

#
#   Update of old IPAC/IpSpeaker UCI configuration to suit new versions
#   This script will be started by the run-once service
#
#   It will write the log to /var/log/uci-update.log

import uci, json, logging, uuid, os, base64
from barix.web.audio_files import *

MAX_AUDIO_SOURCES = 5
UPDATE_LOG = "/var/log/uci-update.log"

def updateUciConfiguration(maxSources, logger):
    try:
        uciObj = uci.Uci()
        for sourceId in range(1,maxSources + 1):
            try:
                srcType = uciObj.get("flexa_ip_former", f"source{sourceId}", "type")
            except uci.UciExceptionNotFound:
                logger.warning(f"source{sourceId} has no configuration")
                continue
            
            config_json = json.loads(uciObj.get("flexa_ip_former", f"source{sourceId}", "config"))
            
            if srcType == 'rtp':
                """
                Insert new playback mode in RTP uci config when upgrading from older versions: 2.6.1 and 2.6.2
                """
                if "url" in config_json:
                    if "playback_mode" not in config_json:
                        logger.info(f"Old RTP audio source{sourceId} config: {config_json}")
                        playbackConfig = {"playback_mode" : "low_latency"}
                        config_json.update(playbackConfig)
                        uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                        logger.info(f"Insert playback_mode config in RTP audio source{sourceId}")
                        uciObj.commit("flexa_ip_former")

            if srcType == "ic-paging":
                """
                Migrate old IC-paging (versions: 2.7.x) UCI config to new Paging in "ic paging" mode
                """
                uciObj.set("flexa_ip_former",f"source{sourceId}","type","paging")
                logger.info(f"Old IC Paging audio source{sourceId} config: {config_json}")
                newConfig = {"paging_mode": "ic paging","sample_rate":"8000","audio_format":"mulaw","barp_station_id":int(config_json["barp_station_id"])}
                config_json.update(newConfig)
                uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))              
                logger.info(f"Update Paging  audio source{sourceId} parameters")
                uciObj.commit("flexa_ip_former")

            if srcType == "paging":
                """
                Convert barp id to integer
                """
                if type(config_json['barp_station_id']) != int:
                    config_json.update({"barp_station_id":int(config_json["barp_station_id"])})
                    logger.info(f"Convert Paging barp id {config_json['barp_station_id']} to integer")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    logger.info(f"Update Paging  audio source{sourceId} parameters")
                    uciObj.commit("flexa_ip_former")

            if srcType == "sip":
                """
                Migrate old versions (version < 2.9.0) to support new SIP audio latency parameters
                """
                if "snd_rec_latency" not in config_json:
                    logger.info(f"Old SIP audio source{sourceId} config: {config_json}")
                    latencyConfig = {"snd_rec_latency":100,"snd_play_latency":100}
                    config_json.update(latencyConfig)
                    logger.info(f"Insert latency parameters config in SIP audio source{sourceId}")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    uciObj.commit("flexa_ip_former")
                    
                if "reg_timeout" not in config_json and config_json['sip_mode'] != 'peer_to_peer':
                    logger.info(f"Old SIP audio source{sourceId} config: {config_json}")
                    regTimeoutConfig = {"reg_timeout":30, "reg_retry_interval":30}
                    config_json.update(regTimeoutConfig)
                    logger.info(f"Insert register timeout and register retry interval parameters config in SIP audio source{sourceId}")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    uciObj.commit("flexa_ip_former")
                    
                if "call_timeout" not in config_json:
                    logger.info(f"Old SIP audio source{sourceId} config: {config_json}")
                    callConfig = {"call_timeout":0, "stream_timeout":0}
                    config_json.update(callConfig)
                    logger.info(f"Insert call timeout and stream timeout parameters config in SIP audio source{sourceId}")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    uciObj.commit("flexa_ip_former")

                """
                Check if passwd exist: in 2.9.0 we store encripted SIP password in a new uci config
                """
                if "passwd" not in config_json and "password" in config_json:
                    logger.info(f"Old SIP audio source{sourceId} config: {config_json}")
                    passwdConfig = {"passwd": f"{base64.b64encode(config_json['password'].encode('utf-8')).decode('utf-8')}"}
                    config_json.pop("password",None)
                    config_json.update(passwdConfig)
                    logger.info(f"Insert encrypted SIP passwd parameter config in SIP audio source{sourceId}")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    uciObj.commit("flexa_ip_former")

                """
                Support 2.10.0 sip parameters: auto_answer, answer_time, input_gain, half_duplex, hdx_level, hdx_timeout
                """
                if "auto_answer" not in config_json:
                    logger.info(f"Old SIP audio source{sourceId} config: {config_json}")
                    config = {"auto_answer":True, "answer_time":0, "input_gain":50, "half_duplex":False, "hdx_level": 0.05, "hdx_timeout": 600}
                    config_json.update(config)
                    logger.info(f"Insert auto_answer, answer_time and input_gain, half_duplex, hdx_level, hdx_timeout parameters config in SIP audio source{sourceId}")
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json))
                    uciObj.commit("flexa_ip_former")
                    
            if srcType != "sip" and srcType != "informacast" and srcType != "syn-apps":
                """
                Migrate old version (version < 2.9.0) to support equalizer configuration
                """
                if "equalizer" not in config_json:
                    logger.info(f"Old audio source{sourceId} - {srcType} - config: {config_json}")
                    equalizerConfig = {'equalizer': {"enable": "false", "config": [{"name": "LOW", "f": "80", "t": "q", "w": "1", "g": "0"},{"name": "LOW-MID", "f": "500", "t": "q", "w": "1", "g": "0"},{"name":"HI-MID", "f": "2000", "t": "q", "w": "1", "g": "0"}, {"name": "HI", "f": "4000", "t": "q", "w": "1", "g": "0"}]}}
                    config_json.update(equalizerConfig)
                    uciObj.set("flexa_ip_former",f"source{sourceId}","config",json.dumps(config_json).replace('\\',''))
                    logger.info(f"Insert equalizer config in {srcType} audio source{sourceId}")
                    uciObj.commit("flexa_ip_former")
                    
                        
    except Exception as e:
        logger.error(f"Exception updating UCI config: {e}")
    

def createAudioFilesID():
    """
    Create new fileIDs for audio files that were updated by older IPAC versions
    """
    audioFolder = "/mnt/data/AudioFiles"
    configFolder = "/mnt/data/config"
    audioJsonFile = "/mnt/data/config/audioFiles.json"
    
    try:
        if os.path.isfile(audioJsonFile):
            logger.info(f"{audioJsonFile} file exist - fileid is already been used.. so nothing to do")
            return
        
        if not os.path.isdir(audioFolder):
            logger.info(f"{audioFolder} folder does not exist... create it")
            os.mkdir(audioFolder)
            return
        
        if len(os.listdir(audioFolder)) == 0:
            logger.info(f"No audio files were found in folder:{audioFolder}")
            return

        audioFilesList = []
        for filename in os.listdir(audioFolder):
            logger.info(f"Found audio file:{filename}")
            audioFile = {}
            fileID = uuid.uuid4().hex
            audioFile['file_id'] = fileID
            audioFile['filename'] = filename
            audioFile['notes'] = ""
            audioFile['duration'] = getAudioFileDuration(os.path.join(audioFolder, filename))
            audioFile['size'] = os.path.getsize(os.path.join(audioFolder, filename))
            #rename filename to fileid
            os.rename(os.path.join(audioFolder, filename),os.path.join(audioFolder, fileID))
            audioFilesList.append(audioFile)

        if not os.path.isdir(configFolder):
            logger.info(f"{configFolder} folder does not exist... create it")
            os.mkdir(configFolder)

        with open(audioJsonFile,'w') as f:
            logger.info(f"Create {audioJsonFile}")
            f.write(json.dumps(audioFilesList))
            
    except Exception as e:
        logger.error(f"Exception creating audio files IDs: {e}")

if __name__ == '__main__':

    logging.basicConfig(level= logging.INFO, filename=UPDATE_LOG, format='%(asctime)s  %(levelname)s %(message)s')
    logger = logging.getLogger(__name__)
    logger.info("Start Checking for Old UCI configuration")

    updateUciConfiguration(MAX_AUDIO_SOURCES,logger)
    createAudioFilesID()
    
    logger.info("Finish!")
